unit htmlhelp;

(***************************************)
(* Copyright (C) 2003, SHIRAISHI Kazuo *)
(***************************************)


interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QComCtrls, QStdCtrls, QExtCtrls;

type
  THelpForm = class(TForm)
    Browser: TTextBrowser;
    Bevel1: TBevel;
    ContentsBtn: TButton;
    BackBtn: TButton;
    ForwardBtn: TButton;
    CopyBtn: TButton;
    Edit1: TEdit;
    SearchBtn: TButton;
    CloseBtn: TButton;
    FontBtn: TButton;
    FontDialog1: TFontDialog;
    procedure ForwardBtnClick(Sender: TObject);
    procedure BackBtnClick(Sender: TObject);
    procedure BrowserTextChanged(Sender: TObject);
    procedure ContentsBtnClick(Sender: TObject);
    procedure SearchBtnClick(Sender: TObject);
    procedure CopyBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure Edit1ReturnPressed(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
     PathList:TStringList;
    procedure SetBrowser(link:string);
  public
    { Public declarations }
  end;

var
  HelpForm: THelpForm;

implementation

uses
  HelpIntfs,
  listbox,base;

{$R *.xfm}

var
  CurrentPos:string;
  KeyWordRec:TStringList;

function HtmName(s:string):string;
var
  p:integer;
begin
  p:=pos('#',s);
  if p=0 then p:=length(s)+1;
  result:=copy(s,1,p-1);
end;

function Anchor(s:string):string;
var
  p:integer;
begin
  p:=pos('#',s);
  if p=0 then
    result:=''
  else
    result:=copy(s,p+1,length(s)-p);
end;



procedure THelpForm.SetBrowser(link:string);
var
   fname:string;
begin
   CurrentPos:=Link;
   fname:= ExtractfilePath(Application.ExeName)+htmName(link);
{$IFDEF WIN32} //Delphi6TTextBrowser.LoadFromFileϤޤȤ˻Ȥʤ
  with TTextViewer.Create(self) do
  begin
      LoadFromFile(fname);
      Browser.text:=text;
      free;
  end;
{$ELSE}
  Browser.FileName:=fname;
{$ENDIF}
  Browser.ScrolltoAnchor(Anchor(link));
  HelpForm.Show;
end;


type
  TKeywordHelp = class(TInterfacedObject, ICustomHelpViewer)
  private
    FViewerID : Integer;
    FHelpManager : IHelpManager;
  public
    { ICustomHelpViewer }
    function GetViewerName: string;
    function UnderstandsKeyword(const HelpString: String): Integer;
    function GetHelpStrings(const HelpString: String): TStringList;
    function CanShowTableOfContents: Boolean;
    procedure ShowHelp(const HelpString: String);
    procedure ShowTableOfContents;
    procedure NotifyID(const ViewerID: Integer);
    procedure SoftShutDown;
    procedure ShutDown;
    property HelpManager: IHelpManager read FHelpManager write FHelpManager;
    property ViewerID: Integer read FViewerID;
  end;


function TKeywordHelp.CanShowTableOfContents: Boolean;
begin
  Result := FileExists(FHelpManager.GetHelpFile);
end;

function TKeywordHelp.GetHelpStrings(const HelpString: String): TStringList;
begin
  Result := TStringList.Create;
  Result.Add(HelpString);
end;

function TKeywordHelp.GetViewerName: String;
begin
  Result := 'BASIC Help';
end;

procedure TKeywordHelp.NotifyID(const ViewerID: Integer);
begin
  FViewerID := ViewerID;
end;

function LinkSelect(const HelpString: String):string;
var
   i:integer;
   s:string;
begin
  s:=KeyWordRec.Values[HelpString];
  if (s='')then
       result:='basic.htm'
  else if s[1]<>'"' then
       result:=s
  else
   begin
     with TStringList.Create do
     begin
       CommaText:=s;
       with TlistForm.create(HelpForm) do
        begin
          RadioGroup1.Caption:=HelpString;
          for i:=0 to (count-1) div 2 do
                RadioGroup1.items.add(strings[i*2]);
          RadioGroup1.ItemIndex:=0;
          RadioGroup1.SetFocus;
          ShowModal;
          result:=Strings[RadioGroup1.ItemIndex *2 +1];
          Free;
        end;
       Free;
      end;
   end;
end;

procedure TKeywordHelp.ShowHelp(const HelpString: String);
begin
  HelpForm.SetBrowser(LinkSelect(HelpString));
end;

procedure TKeywordHelp.ShowTableOfContents;
begin
   ShowHelp('')
end;

procedure TKeywordHelp.ShutDown;
begin
end;

procedure TKeywordHelp.SoftShutDown;
begin
  HelpForm.Close
end;

function TKeywordHelp.UnderstandsKeyword(const HelpString: String): Integer;
begin
  result:=1
end;

var
  HelpContexts:array [0..999] of string ;
  ALinks:TStringList;

procedure InitHelpContexts;
var
   F:text;
   n:integer;
   s:string;
   p:integer;
   c:integer;

begin
   assignFile(F,ChangeFileExt(Application.ExeName,'.hcx'));
{$I-}
   Reset(F);
   if IOResult=0 then
     begin
       while not EOF(F) do
         begin
           Readln(F,s);
           p:=pos(' ',s);
           if p>0 then
             begin
               val(copy(s,p,maxint),n, c);
               if (n>=0) and (n<=999) then HelpContexts[n]:=copy(s,1,p-1);
             end;
         end;
       Close(F);
     end;
{$I+}
end;

procedure InitALinks;
begin
  ALinks:=TStringList.create;
  ALinks.LoadFromFile(ChangeFileExt(Application.ExeName,'.hlk')) ;
end;

type TExtendedHelp=class(TkeywordHelp,IExtendedHelpViewer)
  private
  public
   function UnderstandsContext(const ContextID: Integer; const HelpFileName: String): Boolean;
   procedure DisplayHelpByContext(const ContextID: Integer; const HelpFileName: String);
   function UnderstandsTopic(const Topic: String): Boolean;
   procedure DisplayTopic(const Topic: String);
  end;

function TExtendedHelp.UnderstandsContext(const ContextID: Integer; const HelpFileName: String): Boolean;
begin
 result:=  (ContextID>=0) and (ContextID<=999) 
end;


procedure TExtendedHelp.DisplayHelpByContext(const ContextID: Integer; const HelpFileName: String);
begin

     HelpForm.SetBrowser('html/'+ALinks.Values[HelpContexts[ContextId]]);
end;

function TExtendedHelp.UnderstandsTopic(const Topic: String): Boolean;
begin
 result:=false
end;

procedure TExtendedHelp.DisplayTopic(const Topic: String);
begin
end;


var
  HelpViewer: TExtendedHelp;

procedure THelpForm.ForwardBtnClick(Sender: TObject);
begin
  Browser.Forward;
end;

procedure THelpForm.BackBtnClick(Sender: TObject);
begin
  Browser.Backward;
end;


procedure InitKeyWordRec;
var
   F:text;
   s:string;
begin
   KeywordRec:=TStringList.create;
   KeywordRec.CaseSensitive:=false;
   AssignFile(F,ChangeFileExt(Application.ExeName,'.hkw'));
{$I-}
   Reset(F);
   if IOResult=0 then
     begin
       while not EOF(F) do
         begin
           Readln(F,s);
           KeywordRec.add(s);
         end;
       close(F);
     end;
{$I+}
end;


procedure THelpForm.ContentsBtnClick(Sender: TObject);
begin
    HelpViewer.ShowTableOfContents
end;

procedure THelpForm.SearchBtnClick(Sender: TObject);
begin
      HelpViewer.ShowHelp(trim(Edit1.text));
end;

procedure THelpForm.CopyBtnClick(Sender: TObject);
begin
       Browser.CopyToClipboard
end;

procedure THelpForm.CloseBtnClick(Sender: TObject);
begin
   Close
end;

procedure THelpForm.BrowserTextChanged(Sender: TObject);
begin
  BackBtn.Enabled := Browser.CanGoBackward;
  ForwardBtn.Enabled := Browser.CanGoForward;
end;

procedure THelpForm.Edit1ReturnPressed(Sender: TObject);
begin
      SearchBtnClick(Sender);
end;

procedure THelpForm.FormCreate(Sender: TObject);
begin
  with Font do
    begin
     Height:=Height+2;
     CharSet := fcsSet_Ja;
    end;
  with TMyIniFile.create('Help') do
    begin
      RestoreFont(Font);
      free
    end;
  PathList:=TStringList.Create;
  PathList.Add( Extractfilepath(Application.ExeName));
  PathList.Add( ExtractfilePath(Application.ExeName)+'html/');
  Browser.Factory.FilePath:=PathList;
  Browser.Factory.RegisterMimeType('htm','text/html;charset=eucJP');
end;

procedure THelpForm.FontBtnClick(Sender: TObject);
begin
    FontDialog1.Font:=Font;
    if FontDialog1.execute then
        Font:=FontDialog1.Font;
    Font.CharSet := fcsSet_Ja;
end;

procedure THelpForm.FormDestroy(Sender: TObject);
begin
     with TMyIniFile.create('Help') do
         begin
             StoreFont(Font);
             free
         end;

end;

initialization
    Application.HelpFile:=ChangeFileExt(Application.ExeName,'.htm');
    Application.HelpType:=htContext ;

  HelpViewer := TExtendedHelp.Create;
  HelpIntfs.RegisterViewer(HelpViewer, HelpViewer.FHelpManager);
  InitKeyWordRec;
  InitHelpContexts;
  InitALinks;

finalization
   ALinks.Free;
   KeywordRec.free;
   with HelpViewer do
       if Assigned(FHelpManager) then FHelpManager.Release(FViewerID);

end.

